/*
 * kstart.h Part of the KDE project.
 *
 * SPDX-FileCopyrightText: 1997-2000 Matthias Ettrich <ettrich@kde.org>
 * SPDX-FileCopyrightText: David Faure <faure@kde.org>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 *
 */

#ifndef KSTART_H
#define KSTART_H

#include <QObject>

class KStart : public QObject
{
    Q_OBJECT

public:
    KStart();
};

#endif
